SET PROCEDURE TO wwScripting ADDITIVE

*************************************************************
DEFINE CLASS wwScripting AS Custom
*************************************************************
*: Author: Rick Strahl
*:         (c) West Wind Technologies, 2004
*:Contact: http://www.west-wind.com
*:Created: 09/24/2004
*************************************************************
#IF .F.
*:Help Documentation
*:Topic:
Class wwScripting 

*:Description:
This class provides end user applications with the ability to
dynamically evaluate and execute templates. It provides both
a TextMerge expression evaluator and a fully capable script
parser using Visual FoxPro language code. Both mechanisms
are based on the ASP <%=  %> script tag expressions.

*:Remarks
This class requires VFP 8 or later

*:ENDHELP
#ENDIF
************************************************************************

*** Error Message if a failure occurs
cErrorMsg = ""

*** Error flag that can and should be checked after running a script or template
lError = .F.

*** A VFP 8/9 Exception object
oException = null

*** If .t. doesn't delete the generate VFP source code after compiling
lSaveVfpSourceCode = .F.

*** If .T. pops up an editor window for editing the error in the code
lEditErrors = .f.

*** Path where our compiled FXP and WCT files are stored
cCompiledPath = ""

PROTECTED cCurrentTemplate
*** Internal property that holds the currently processing template
cCurrentTemplate = ""

*** Not used at this time
lUnloadAfterRunning = .F.

*** Not used at this time
lNoVersionCheck = .f.

************************************************************************
FUNCTION AspTextMerge(lcAspCode,llIsFile)
****************************************
#IF .F.
*:Help Documentation
*:Topic:
wwScripting :: AspTextMerge

*:Description:
This method is a wrapper around the VFP TextMerge function
to provide template expansion of <%= %> syntax. It also
provides proper error handling.


*:Parameters:
<<b>>lcASPCode<</b>>
The code that is to be textmerged. This code should be
TEXT with embedded <%= %> tags for expressions TO be
evaluated. Note that any expressions must be in scope
(ie. vars need to be PRIVATE or PUBLIC).

You can also pass a filename here if llIsFile is set to .t.

<<b>>llIsFile<</b>>
IF .t. specifies that the first parameter indicates a filename.

*:Returns:
Expanded text. Check lError for failures.

*:Remarks:

*:SeeAlso:

*:Keywords:

*:ENDHELP
#ENDIF
************************************************************************
LOCAL lcASPCode, lcResult

this.lError = .f.

IF llIsFile
   lcASPCode = FILETOSTR(lcAspCode)
ENDIF

*** Must convert to 2 characters from <%=
lcASPCode = STRTRAN(lcAspCode,"<%=","<%")

lcResult = ""
loException = ""
TRY 
  lcResult = TEXTMERGE(lcASPCode,.f.,"<%","%>") 
CATCH TO loException
   this.oException = loException
   this.cErrorMsg = this.oException.Message
   this.lError = .t.
ENDTRY

RETURN lcResult
ENDFUNC


************************************************************************
FUNCTION RenderAspScript(lcTemplate)
****************************************
#IF .F.
*:Help Documentation
*:Topic:
wwScripting :: RenderASPScript

*:Description:
Renders an ASP style page as a compiled VFP program. This method
handles creation of the VFP source code from the script file,
compilation and checking for updates of the file.

Parsing works using ASP style syntax:

<%= Expression  %>

<%
VFP Code Block
%>

The script page gets passed a Response object which you can
USE to send output directly into the output stream. 
 
*:Parameters:
lcTemplate
Name of the file on disk to parse

*:Returns:
Evaluated result from the script code.
*:ENDHELP
#ENDIF
************************************************************************
LOCAL loException, lcOutFile, lcFile, lcFXPFile, lcWCTFile, lcVFPCode,;
      lcFileName, lcFXPFile, lcWCTFile, llNeedToCompile

IF EMPTY(THIS.cCompiledPath)
   this.cCompiledPath = JUSTPATH(lcTemplate) + "\"
ELSE
   this.cCompiledPath = ADDBS(THIS.cCompiledPath)   
ENDIF

this.cCurrentTemplate = lcTemplate

*** Figure out our filenames to use
lcFileName = JUSTFNAME(lcTemplate)
lcFXPFile =  FORCEEXT(THIS.cCompiledPath + lcFileName,"FXP")

this.lError = .f.

llNeedToCompile = .T.
TRY
   *** This catches both fxp file not existing and being out of date
   *** Slightly more efficient than checking file, and the dates
   llNeedToCompile = FDATE(lcFXPFile,1) < FDATE(lcTemplate,1)
CATCH
   llNeedToCompile = .t.
ENDTRY

*** Create and compile the file only if it doesn't exist
*** or the timestamp of the FXP is older than the source file
*** NOTE: Must use SYS(2000) instead of file so we don't look for files in the EXE!
IF  llNeedToCompile
   IF !this.ConvertAndCompileScript(lcTemplate)
      RETURN this.AspScriptErrorResponse()
   ENDIF
ENDIF

*** We're going to TextMerge to a temporary file
lcOutFile = this.cCompiledPath + "WCS"+Sys(2015)+ TRANSFORM(Application.ProcessId) + ".TMP"
Response = CREATEOBJECT("wwScriptingResponse",lcOutfile) 

loException = null
TRY 
   *** Just execute the PRG File
   DO &lcFxpFile
CATCH TO loException
   this.lError = .t.
   this.oException = loException
   this.cErrormsg = this.oException.Message
ENDTRY   

IF THIS.lError
   RETURN this.AspScriptErrorResponse()
ENDIF


RETURN Response.GetOutput()
ENDFUNC


************************************************************************
FUNCTION ConvertAndCompileScript( lcTemplate as String) 
****************************************
#IF .F.
*:Help Documentation
*:Topic:
wwScripting :: ConvertAndCompileScript

*:Description:
This method takes a script file and converts it to a VFP file and
compiles it.

This method allows your code to pre-compile pages if you choose.
*:Parameters:
lcTemplate
The template to convert.
*:Returns:
.T. of .f.
*:Remarks:

*:SeeAlso:

*:Keywords:

*:ENDHELP
#ENDIF
************************************************************************
LOCAL lcWCTFile, lcFXPFile, lcFileName

lcFileName = JUSTFNAME(lcTemplate)
lcFXPFile =  FORCEEXT(THIS.cCompiledPath + lcFileName,"FXP")
lcWCTFile =  FORCEEXT(THIS.cCompiledPath + lcFileName,"WCT")

*** Must create directory if it doesn't exist
IF !ISDIR(this.cCompiledPath)
   MD (this.cCompiledPath)
ENDIF   

IF !FILE(lcTemplate)
   this.cErrorMsg = "ASP Template doesn't exist"
   this.oException = null
   this.lError = .t.
   RETURN .F.
ENDIF      

lcVFPCode = this.ConvertAspToVfpCode( FILETOSTR(lcTemplate))
STRTOFILE( lcVFPCode,lcWCTFile )

loException = null
TRY
   COMPILE ( lcWctFile )

   *** Once compiled we don't need the source code any more
   IF !this.lSaveVfpSourceCode
      ERASE ( lcWCTFile) 
   ENDIF
CATCH TO loException
   this.lError = .t.
   this.oException = loException
   this.cErrorMsg = this.oException.Message
ENDTRY


RETURN .T.
ENDFUNC

************************************************************************
FUNCTION ConvertAspToVFPCode( lcFile AS String) AS String
****************************************
#IF .F.
*:Help Documentation
*:Topic:
wwScripting :: ConvertAspTemplateToVfpCode

*:Description:
Takes an input script file and converts the file into a string 
that contains a VFP program of that script page. The page is
parsed into a PRG that executes several blocks of TEXTMERGE command
to evaluate the code in the page.

*:Parameters:
lcScriptFile
The File to convert

*:Returns:
String containing the parsed source code.

*:Remarks:

*:ENDHELP
#ENDIF
************************************************************************
LOCAL lcFile, lcVFPCode, lcEvalReplace, lcOutFile, llReturnCode, lcEvalCode

*** Start by converting Expressions (<%= %>) 
*** to standard TextMerge characters (<< >>)
lcEvalCode = "x"
DO WHILE !EMPTY(lcEvalCode)
   lcEvalCode = Extract(lcFile,"<%=","%>")
   IF !EMPTY(lcEvalCode)
      lcEvalReplace = "<%=" + lcEvalCode + "%>"
      lcFile = STRTRAN(lcFile,lcEvalReplace,"<<" +ALLTRIM(lcEvalCode) + ">>")
   ENDIF
ENDDO

*** Now let's deal with 'script blocks' by putting TEXT/ENDTEXT
lcFile = STRTRAN(lcFile,"<%",CHR(13)+"ENDTEXT"+CHR(13))
lcVFPCode =;
 "TEXT" + CHR(13) +;
 STRTRAN(lcFile,"%>",CHR(13)+"TEXT"+CHR(13)) + ;
 CHR(13) + "ENDTEXT"  

RETURN lcVFPCode
ENDFUNC
* ConvertPage

************************************************************************
PROTECTED FUNCTION AspScriptErrorResponse
****************************************
#IF .F.
*:Help Documentation
*:Topic:
wwScripting :: ScripterrorResponse

*:Description:
Creates an Error Response page based on the oException object
that might have captured an exception in the script execution.

*:ENDHELP
#ENDIF
************************************************************************
LOCAL lcOutput, lnErrorLine, lnErrorPos, lnLoc, lcFileContent,lcCode

lcOutput = ""

*** Figure out our filenames to use
lcFileName = JUSTFNAME(this.cCurrentTemplate)
lcFXPFile =  FORCEEXT(THIS.cCompiledPath + lcFileName,"FXP")
lcWCTFile =  FORCEEXT(THIS.cCompiledPath + lcFileName,"WCT")

IF !ISNULL(this.oException)
   lcOutput = ;
      [<TABLE WIDTH=100% BGCOLOR="BLACK"><TD ALIGN=CENTER><b><FONT COLOR="WHITE" FACE="VERDANA" SIZE="4">Scripting Error</FONT></b></TD></TABLE><p>]+;
      [<TABLE BGCOLOR=#EEEEEE BORDER=1 CELLPADDING=3 STYLE="Font:normal normal 10pt Verdana">] + CHR(13) + CHR(10) +;
      "<TR><TD ALGIN=RIGHT>Error: </TD><TD>"+FixHtmlForDisplay( this.oException.Message ) + "</TD</TR>" + ;
      "<TR><TD ALGIN=RIGHT>Code:</TD><TD>"+FixHtmlForDisplay( this.oException.LineContents ) + "</TD></TR>" +;
      "<TR><TD ALGIN=RIGHT>Error Number:</TD><TD>"+ TRANSFORM(this.oException.ErrorNo) + "</TD></TR>" +;
      "<TR><TD ALGIN=RIGHT>Line No:</TD><TD>" + TRANSFORM(this.oException.LineNo) + "</TD></TR>" + ;
      "</TABLE><p>"

      IF this.lEditErrors AND this.lSaveVfPSourceCode
           lcCode = this.oException.LineContents
           IF lcCode = "<<"
              lcCode = STRTRAN(lcCode,"<<","")
              lcCode = STRTRAN(lcCode,">>","")
           ENDIF
           lcSourceFile = this.cCurrentTemplate
           IF FILE(  lcSourceFile )
              lcFileContent = FILETOSTR( lcSourceFile)
              lnLoc = ATC(lcCode,lcfileContent)
              IF ATC(lcCode,lcfileContent) = 0
                 MODIFY FILE (lcSourceFile) IN MACDESKTOP NOWAIT
              ELSE
                 MODIFY FILE (lcSourceFile) RANGE (lnLoc),(lnLoc + LEN(lcCode)) IN MACDESKTOP NOWAIT
              ENDIF
           ENDIF
      ENDIF
ELSE
      lcOutput = [<TABLE WIDTH=100% BGCOLOR="BLACK"><TD ALIGN=CENTER><b><FONT COLOR="WHITE" FACE="VERDANA" SIZE="4">Scripting Error</FONT></b></TD></TABLE><p>] + ;
      [<TABLE BGCOLOR=#EEEEEE BORDER=1 CELLPADDING=3 STYLE="Font:normal normal 10pt Verdana">] + ;
      "<TR><TD ALGIN=RIGHT>Error: </TD><TD>"+this.oException.cErrorMsg + "</TD</TR>" + ;
      "</TABLE><p>"
ENDIF

RETURN lcOutput
ENDFUNC
* wwVFPScript :: wwEvalErrorResponse

ENDDEFINE


*************************************************************
DEFINE CLASS wwScriptingResponse AS Custom
*************************************************************
***    Author: Rick Strahl
***            (c) West Wind Technologies, 1997
***   Contact: (541) 386-2087  / rstrahl@west-wind.com
***  Modified: 11/18/97
***
***  Function: Provides basic Response.Write() functionality
***            to inject output into the generated script
***            page.
*************************************************************

*** Custom Properties
cFileName = ""

PROTECTED lOpened
lOpened = .f.

*** Stock Properties

************************************************************************
* wwScriptResponse :: Init
*********************************
***  Function:
***    Assume:
***      Pass:
***    Return:
************************************************************************
FUNCTION Init
LPARAMETERS lcOutputFile, loVFPScript

THIS.cFileName = lcOutputFile
this.Open()

ENDFUNC
* wwScriptResponse :: Init

************************************************************************
* wwScriptResponse :: Destroy
****************************************
***  Function:
***    Assume:
***      Pass:
***    Return:
************************************************************************
FUNCTION Destroy()

IF this.lOpened
   this.GetOutput()  && Close the file and delete
ENDIF   

ENDFUNC
*  wwScriptResponse :: Destroy


************************************************************************
* wwScriptResponse :: Open
****************************************
***  Function:
***    Assume:
***      Pass:
***    Return:
************************************************************************
FUNCTION Open()

SET TEXTMERGE ON
SET TEXTMERGE TO (this.cFileName) NOSHOW
this.lOpened = .t.

ENDFUNC
*  wwScriptResponse :: Open

************************************************************************
* wwScriptResponse :: Write
*********************************
***  Function: Basic Output Method for scripting. Used for direct
***            call or when using <%= <expression %> syntax.
************************************************************************
FUNCTION Write
LPARAMETER lvExpression

\\<<lvExpression>>

ENDFUNC
* Write

FUNCTION SEND
LPARAMETER lvExpression

\\<<lvExpression>>
ENDFUNC

************************************************************************
* wwScriptResponse :: Clear
*********************************
***  Function:
***    Assume:
***      Pass:
***    Return:
************************************************************************
FUNCTION Clear

this.GetOutput()

ENDFUNC
* wwScriptResponse :: Clear

FUNCTION Rewind
THIS.Clear()
ENDFUNC

************************************************************************
* wwScriptResponse :: GetOutput
*********************************
***  Function:
***    Assume:
***      Pass:
***    Return:
************************************************************************
FUNCTION GetOutput

SET TEXTMERGE TO

*** Retrieve the output
lcOutput = filetostr(THIS.cFileName)

*** And immediately close file
ERASE (THIS.cFileName)

this.lOpened = .f.

RETURN lcOutput 
ENDFUNC
* wwScriptResponse :: GetOutput

 
ENDDEFINE
*EOC wwScriptResponse
